@@ -401,6 +401,14 @@ class NameMatchingPolicy {
   NameMatchingPolicy(const NameMatchingPolicy&) = delete;
   void operator=(const NameMatchingPolicy&) = delete;
 };
+
+class StrictNameMatchingPolicy : public NameMatchingPolicy {
+  public:
+    virtual Result FallBackToCommonName(
+        Time notBefore,
+        /*out*/ FallBackToSearchWithinSubject& fallBacktoCommonName) override;
+};
+
 }
 }  // namespace mozilla::pkix
 
